/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "fileManager.h"

FileManager* fileMana = 0;

FileManager::FileManager() {
#ifdef WIN32
	mRootDir = "." ;
#else
	if ( access ( "data/newtPuzzleConfig.data", F_OK ) == 0 )
		mRootDir = "." ;
	else if ( access ( "../data/newtPuzzleConfig.data", F_OK ) == 0 )
		mRootDir = ".." ;
    else
	#ifdef NEWTPUZZLE_DATA_DIR
		mRootDir = NEWTPUZZLE_DATA_DIR ;
	#else
        mRootDir = "/usr/local/share/games/newt-puzzle" ;
	#endif
#endif //win32
	std::cout << "Data files will be fetched from: " << mRootDir.c_str() << std::endl;
}


void FileManager::initDir()
{
	std::string pathname;
	
#ifdef WIN32
	pathname = "./screenshots";
//	mkdir();
	CreateDirectory(pathname.c_str(),NULL);
	
#else
	//if HOME environment variable exists create directory $HOME/.newtpuzzle	
    if(getenv("HOME")!=NULL)
    {
        pathname = getenv("HOME");
        pathname += "/.newtpuzzle";
		mkdir(pathname.c_str(), 0755);
    } else {
		pathname = "./screenshots";
		mkdir(pathname.c_str(), 0755);	
	}
#endif
}  

void FileManager::initNewtStateDir(std::string dirPath) {
#ifdef WIN32
	CreateDirectory(dirPath.c_str(),NULL);
#else
	mkdir(dirPath.c_str(), 0755);	
#endif

}


std::string FileManager::getHomeNewtDir() const {
	std::string dir;
#ifdef WIN32
	dir = ".";
#else
	if(getenv("HOME")!=NULL) {
		dir = getenv("HOME");
	} else {
		dir = ".";
    }
	dir += "/.newtpuzzle";
#endif
	return dir;
} 

std::string FileManager::getScreenshotsDir() const {
	std::string dir;
#ifdef WIN32
	dir = "./screenshots";
#else
	if(getenv("HOME")!=NULL) {
		dir = getenv("HOME");
		dir += "/Desktop";
	} else {
		dir = "./screenshots";
    }
#endif
	return dir;	
}


std::string FileManager::getDataDir() const {
	return mRootDir+"/data";
}

std::string FileManager::getShadersDir() const {
	return mRootDir+"/shaders";
}

std::string FileManager::getFontsDir() const {
	return mRootDir+"/data/fonts";
}

std::string FileManager::getTexturesDir() const {
	return mRootDir+"/data/textures";
}

std::string FileManager::getHightTexturesDir() const {
	return mRootDir+"/data/textures/hightRes";
}

std::string FileManager::getLowTexturesDir() const {
	return mRootDir+"/data/textures/lowerRes";
}

std::string FileManager::getSmallTexturesDir() const {
	return mRootDir+"/data/textures/smallTex";
}
