/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "gameChooser.h"
  
GameChooser::GameChooser() : NewtState("Game Chooser", "GameChooser"), selectedLauncher(NP_NONE) {
	init();	
	mKeyboardOptions = new GameChooserKeyboardOptions(mDirPath + "/keyboardOptions.dat");
}

GameChooser::~GameChooser() {	
}


newtGame GameChooser::getSelectedLauncher() {
	return selectedLauncher;	
}


void GameChooser::initDisplayEnvironments() {
	Display3D *disp3D = new Display3D();
	scene->addDisplay3D(disp3D);

	if(scene->getGraphicOptions()->isGLSLEnabled()) {
		scene->setClearColor(8,0.6);		
		disp3D->setFog(new Fog(GL_LINEAR, 1., 300., 1300., 8, 0.6));
	} else {
		scene->setClearColor(8,0.3);	
		disp3D->setFog(new Fog(GL_LINEAR, 1., 300., 1300., 8, 0.3));
	}
	
	int width = generalGraphics->getScreenWidth();
	int height = generalGraphics->getScreenHeight();
	float fov = 70.;
	
	camera = new Camera(PRISM_INNER_RADIUS*0.8, fov, 5., 1600, 0, 0, width, height);	
	disp3D->setCamera(camera);
	Vector3 positionCam = Vector3(0., EYE_HEIGHT, 400.);
	camera->setPositionAndAxis(positionCam, Vector3(0.,100.,0.),Vector3(0.,1.,0.));
}

void GameChooser::createObjects() {
	Display3D *disp3D = scene->getDisplay3D(0);
	
	/* creation of the floor */
	materialFloor = new Material("newtMozaic", 14);
	materialFloor->setAmbientDiffuseColor(0.8,1.);	
	materialFloor->addTexture("newtMozaic.png", false, 0, COLOR_MAP, GL_CLAMP, true, scene->getGraphicOptions());
	materialFloor->addTexture("newtMozaicNormalMap.png", false, 0, NORMAL_MAP, GL_CLAMP, true, scene->getGraphicOptions());
	materialFloor->addTexture("newtMozaicGlossMap.png", false, 0, GLOSS_MAP, GL_CLAMP, true, scene->getGraphicOptions());
	materialFloor->setSpecularColor(-0.8, 30.);
	

	materialWall = new Material("whiteTest", 14);
	materialWall->setSpecularColor(-0.9, 50.);

	
	prism = new PrismWithPosition(12, PRISM_HEIGHT, 1, PRISM_INNER_RADIUS, 1, 5., 2, 0.01, BASE_NONE, BASE_FLAT, false, materialWall, NULL, materialFloor);
	prism->setPosition(Vector3(0.,PRISM_Y_POS,0.), Vector3(0.,0.,1.), 0.);	
	//prism->setPosition(Vector3(0.,PRISM_Y_POS,0.), Vector3(0.,0.,1.), 90.);	
	disp3D->addEntity(prism); 
	
	if(scene->getGraphicOptions()->isGLSLEnabled()) {
		std::string skyFilename = fileMana->getTexturesDir() + "/smallTex/sky.png";
		std::string skyNightFilename = fileMana->getTexturesDir() + "/smallTex/skyNight.png";
		sky = new Sky(skyFilename,skyNightFilename);
		disp3D->setSky(sky);
		
		SkyPlane *skyPlane = new SkyPlane("skyPlaneCloudsOnly.png", 1500., 200., true);
		disp3D->setSkyPlane(skyPlane);		
		
	} else {
		SkyPlane *skyPlane = new SkyPlane("skyPlaneTex.png", 1500., 200., false);
		disp3D->setSkyPlane(skyPlane);
	}
		
	
	//Mesh *rectangleSea = new RectangleMesh( 1000., 2, 0., 1000., 2, 0.);
	//sea = new WaterPlane(Color(0.2, 0.4, 0.7, 1.0), 0., rectangleSea, Vector3(0.,0.,0.));
	//disp3D->setWaterPlane(sea);
	
	
	Material *iceMaterial = new Material("ice", 8);
	iceMaterial->setAmbientDiffuseColor(0.,0.5);
	iceMaterial->setSpecularColor(0.6, 30.);
	iceMaterial->addTexture("ice.png", true, 0, COLOR_MAP, GL_REPEAT, 3., 3., true, scene->getGraphicOptions());
	iceMaterial->addTexture("iceGloss.png", true, 0, GLOSS_MAP, GL_REPEAT, 3., 3., true, scene->getGraphicOptions());
	
	Mirror *icePlane = new Mirror(iceMaterial, 3300., 10, 3300., 10, Vector3(0.,0.,0.), Vector3(1.,0.,0.), -90.);
	disp3D->addMirror(icePlane);	
	
	SphereMesh *launcherSphere = new SphereMesh(15., 12, 8, false);
	
	GameLauncher *ticTacToeLauncher = new GameLauncher (NP_TIC_TAC_TOE, launcherSphere, scene->getGraphicOptions());
	ticTacToeLauncher->setPosition(Vector3(80.,80.,0.));
	disp3D->addEntity(ticTacToeLauncher);
	launchers.push_back(ticTacToeLauncher);

	GameLauncher *scoreFourLauncher = new GameLauncher (NP_SCORE_FOUR, launcherSphere, scene->getGraphicOptions());
	scoreFourLauncher->setPosition(Vector3(-80.,80.,0.));
	disp3D->addEntity(scoreFourLauncher);	
	launchers.push_back(scoreFourLauncher);
	
	/*GameLauncher *keplerBubbleLauncher = new GameLauncher (NP_KEPLER_BUBBLE, launcherSphere, scene->getGraphicOptions());
	keplerBubbleLauncher->setPosition(Vector3(0.,0.,0.));
	disp3D->addEntity(keplerBubbleLauncher);	
	launchers.push_back(keplerBubbleLauncher);*/
	
	
	EntityWithPosition *cylinder;
	float cylinderHeight = PRISM_INNER_RADIUS*0.9;
	
	cylinder = new EntityWithPosition(materialWall,
									  new CylinderMesh(cylinderHeight, 1, 1.5, 6, BASE_NONE, BASE_DOME, 3, false),
									  true,
									  Vector3(-PRISM_INNER_RADIUS/2., cylinderHeight/2., 1.4*PRISM_INNER_RADIUS), Vector3(0., 0., 1.), 90.);
	disp3D->addEntity(cylinder);

	cylinder = new EntityWithPosition(materialWall,
									  new CylinderMesh(cylinderHeight, 1, 1.5, 6, BASE_NONE, BASE_DOME, 3, false),
									  true,
									  Vector3(PRISM_INNER_RADIUS/2., cylinderHeight/2., 1.4*PRISM_INNER_RADIUS), Vector3(0., 0., 1.), 90.);
	disp3D->addEntity(cylinder);
	
	
	EntityWithPosition *rectangle;
	float rectangleHeight = PRISM_INNER_RADIUS / 4.;
	float texOffset = 0.01;
	
	
	Material *materialNewtPuzzle = new Material("newt-puzzle", 14);
	materialNewtPuzzle->addTexture("Newt-Puzzle.png", false, 0, COLOR_MAP, GL_REPEAT, true, scene->getGraphicOptions());
	materialNewtPuzzle->addTexture("Newt-Puzzle_glossMap.png", false, 0, GLOSS_MAP, GL_REPEAT, true, scene->getGraphicOptions());
	
	materialNewtPuzzle->setSpecularColor(0.6, 30.);
	
	rectangle = new EntityWithPosition(materialNewtPuzzle,
									   new RectangleMesh(PRISM_INNER_RADIUS, 1, 0.,1., rectangleHeight,1, texOffset, 1.-texOffset),
									   true,
									   Vector3(0., cylinderHeight-rectangleHeight/2.,1.4*PRISM_INNER_RADIUS + 1.), Vector3(0.,1.,0.),0.);
	disp3D->addEntity(rectangle);

	rectangle = new EntityWithPosition(materialNewtPuzzle,
									   new RectangleMesh(PRISM_INNER_RADIUS, 1, 0.,1., rectangleHeight,1, texOffset, 1.-texOffset),
									   true,
									   Vector3(0., cylinderHeight-rectangleHeight/2.,1.4*PRISM_INNER_RADIUS - 1.), Vector3(0.,1.,0.),180.);
	disp3D->addEntity(rectangle);	
	
	cylinder = new EntityWithPosition(materialNewtPuzzle,
									  new RoundEdgeMesh(PRISM_INNER_RADIUS, 1, 0., 0., 1., M_PI, 5, 1.-texOffset, 1.+texOffset, false),
									  true,
									  Vector3(0., cylinderHeight,1.4*PRISM_INNER_RADIUS), Vector3(1.,0.,0.),-90.);
	disp3D->addEntity(cylinder);
									  
	cylinder = new EntityWithPosition(materialNewtPuzzle,
									  new RoundEdgeMesh(PRISM_INNER_RADIUS, 1, 0., 1., 1., M_PI, 5, texOffset, -texOffset, false),
									  true,
									  Vector3(0., cylinderHeight-rectangleHeight,1.4*PRISM_INNER_RADIUS), Vector3(1.,0.,0.),90.);
	disp3D->addEntity(cylinder);	
	
}

void GameChooser::createLights() {
	Display3D *disp3D = scene->getDisplay3D(0);	
	
	Vector3 lightPosition = Vector3(0.,80.,0.);	
	//disp3D->addLightPoint(new LightPoint(0, 2, lightPosition, 2., 50.));
	
	lightPosition = Vector3(150.,100.,0.);	
	lightPosition = Vector3(-350.,100.,100.);	
	
	//disp3D->addLightPoint(new LightPoint(0, 2, lightPosition, 1., 100.));
	
	
	/*LightPoint *lightLauncher;
	for(int i=0;i<launchers.size();i++) {
		lightLauncher = new LightPoint(6, 2, Vector3(0.,0.,0.), 0.5, 80.);
		lightsLaunchers.push_back(lightLauncher);
		disp3D->addLightPoint(lightLauncher);
	}*/
	
	float phi = 0.;
	float theta = 0.;
	
	Vector3 spotPosition = Vector3(0., 300., 0.);
	Vector3 spotDirection = Vector3(sin(theta)*cos(phi), -cos(theta), sin(theta)*sin(phi));
	spot = new LightSpot(14, 2, spotPosition, spotDirection, 30., 20., 1., 300);
	//disp3D->addLightSpot(spot);
	
	
	theta = M_PI/4.;
	Vector3 sunDirection = Vector3(0., cos(theta),sin(theta));
	sun = new LightDir(0, -0.1, 2, sunDirection);
	disp3D->addLightDir(sun);
	
	//sky->setSunDirection(sunDirection);
	
}


void GameChooser::updateAnimations(double currentTime, double intervalTime) {
	/*if(spot->getStatus()) {
		float phi = currentTime/9.*M_PI;
		float theta = (1.-cos(currentTime/8.*M_PI))/10.*M_PI;
		
		Vector3 spotDirection = Vector3(sin(theta)*cos(phi), -cos(theta), sin(theta)*sin(phi));	
		spot->setSpotDirection(spotDirection);
	}*/
	
	float lambdaSun = currentTime * M_PI;

	scene->getDisplay3D(0)->setAmbientColors(Color(0.04, 0.08, 0.08, 1.), Color(0.03, 0.06, 0.12,1.), Vector3(0.,1.,0.));
	
	float phiLauncher;
	float radius;
	float hLauncher;
	int nbGames = launchers.size();
	
	for(int i=0;i<nbGames;i++) {
		float a = (float) i * 2. / (float) nbGames;
		phiLauncher = (currentTime/15. + a )*M_PI;
		radius = PRISM_INNER_RADIUS * (3. + 2. * cos(currentTime / 3. + a))/4.;
		hLauncher = 90. * (3. + sin(currentTime * 2./3. + a)) /4.;
		launchers[i]->setPosition(Vector3(radius * cos(phiLauncher), hLauncher , radius * sin(phiLauncher)));	
		//lightsLaunchers[i]->setPosition(Vector3(radius * cos(phiLauncher), hLauncher , radius * sin(phiLauncher)));			
	}
	
	int screenWidth = generalGraphics->getScreenWidth();
	int screenHeight = generalGraphics->getScreenHeight();
	
	int hit = scene->getHit(launchers.size() * 4, screenWidth / 2, screenHeight / 2, (float) screenWidth / 4., (float) screenWidth / 4., 150.);	

	
	if(hit != selectedLauncher) {
		if(hit != -1) {
			selectedLauncher = (newtGame) hit;
			std::cout << launchers[hit - 1]->getName() << std::endl;
		} else {
			selectedLauncher = NP_NONE;
		}
	}
}


void GameChooser::quit() {

	
}


bool GameChooser::keyboardEvent(SDLKey key, double timeInterval) {
	float speed = 150.;
	
	if(key == generalKeyboardOptions->getKey("quit_key")) {
		mQuit = true;
		return false;
	}
	
	if(key==mKeyboardOptions->getKey("forward_key")) {
		move(Vector3(0.,0.,timeInterval*speed));
		return true;
	}
	if(key==mKeyboardOptions->getKey("back_key")) {
		move(Vector3(0.,0.,-timeInterval*speed));
		return true;
	}		
	if(key==mKeyboardOptions->getKey("left_key")) {
		move(Vector3(timeInterval*speed,0.,0.));
		return true;
	}
	if(key==mKeyboardOptions->getKey("right_key")) {
		move(Vector3(-timeInterval*speed,0.,0.));
		return true;
	}	

	
}


void GameChooser::move(Vector3 step) {
	Vector3 position = camera->getPosition();
	position = Vector3(position.x, 0., position.z);
	Vector3 newPosition = camera->getTranslatedPosition(step, Vector3(0.,1.,0.));
	newPosition = Vector3(newPosition.x, 0., newPosition.z);
	
	float prismRadius2 = PRISM_INNER_RADIUS*PRISM_INNER_RADIUS * 0.99;
	float prismRadiusBis2 = prismRadius2 * 1.01;
	
	float distNewPos2 = newPosition.getNorm2();
	
	float yCam;
	
	if(distNewPos2 > DIST_MAX_2) {
		newPosition = position;
		yCam = EYE_HEIGHT;		
	
	} else {
	
		if(distNewPos2 < prismRadius2) {
			yCam = EYE_HEIGHT + PRISM_HEIGHT/2. + PRISM_Y_POS;
		} else {
			if(distNewPos2 > prismRadiusBis2) {
				yCam = EYE_HEIGHT;		
			} else {
				yCam = EYE_HEIGHT + (PRISM_HEIGHT/2. + PRISM_Y_POS) * (prismRadiusBis2 - distNewPos2) /(prismRadiusBis2 - prismRadius2);
				newPosition = 0.05 * newPosition + 0.95 * position;
			}
		}
	}
	
	camera->setPosition(Vector3(newPosition.x, yCam, newPosition.z));
}


	
void GameChooser::launchGame() {

}



void GameChooser::mouseMotion(int xOld, int yOld, int xNew, int yNew, bool left, bool middle, bool right) {
	camera->rotate(STEP_ANGLE*0.04*(xOld - xNew), STEP_ANGLE*0.02*(yOld -yNew));
}

void GameChooser::leftClick(int x, int y) {

}

void GameChooser::rightClick(int x, int y) {

}

void GameChooser::middleClick(int x, int y) {

}

void GameChooser::mouseWheel(int w) {

}

