/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _VIRTUAL_GRIDS_H_
#define _VIRTUAL_GRIDS_H_

#include <vector>
#include "../newtEngine/glUtil.h"
#include "../newtEngine/material.h"

#include "../variousEntities/sphere.h"
#include "../variousEntities/cylinder.h"


#include "../newtEngine/scene.h"
#include "entityTab.h"
#include "entityPlane.h"

#define ANGLE_DIAG_CUBE 54.735610

#define SPHERES_RADIUS 5.
#define CYLINDER_RADIUS 0.8
#define GRID_INTERVAL 25.



class Grid : public EntityWithPosition 
{
public:
	Grid(int size, std::vector<int> colors, float interval, float cylinderRadius);
	virtual ~Grid();
					
	int getSize();
		
	virtual int getNbSpheres() = 0;
				
	virtual void reinit() = 0;
	
	virtual int incScore(int player, int position) = 0;
	
	void selectEntityTTT(int player, int position);
	void selectEntitySF(int player, int hit);
		
	//return true if the current player finish his turn
	bool changePlayerTTT(int player, int position);
	bool changePlayerSF(int player, int hit, int position);
		

protected:	
	int mSize;
	float mInterval;
	int selectedEntity;
	
	std::vector<Material*> materials;
		
	int mainEntitiesIndex, verticalRodsIndex;
			
};


class GridCube: public Grid
{
public:
	GridCube(int size, std::vector<int> colors, float interval, float cylinderRadius);
	virtual ~GridCube();
	
	int getNbSpheres();
		
	int incScore(int player, int position);
		
	virtual int updateAxis(int player, int i, int j, int k) = 0;
	int updateDiag(int player, int i, int j, int k);
	int updateDiagCube(int player, int i, int j, int k);
	
protected:	
		
	CylinderMesh *cylinderDiag;
	CylinderMesh *cylinderDiagCube;
		
};






	
#endif // _VIRTUAL_GRIDS_H_
