/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "newtPuzzle.h"

#include "window.h"

NewtPuzzle* newt = 0;

NewtPuzzle::NewtPuzzle() {
	stateName = NP_CHOOSER;
	chooser = new GameChooser();
	currentNewtState = chooser;
	window_showCursor (false);
}

NewtPuzzle::~NewtPuzzle() {
	//delete gameChooser;
}



Game* NewtPuzzle::getCurrentGame() {
	return currentGame;	
}

/*Menu* NewtPuzzle::getCurrentMenu() {
	return currentMenu;	
}*/

NewtState* NewtPuzzle::getNewtCurrentState() {
	return currentNewtState;
}


newtGame NewtPuzzle::getNameGame() {
	return nameGame;
}

newtStateName NewtPuzzle::getStateName() {
	return stateName;
}



void NewtPuzzle::launch(newtGame selectedLauncher) {
	
	if(selectedLauncher != NP_NONE) {
		scene->clear();
		delete chooser;
		chooser = NULL;	
	
		switch(selectedLauncher){

			case NP_TIC_TAC_TOE:
				currentGame = new TicTacToeCube ();
				window_showCursor(true);
				window_captureCursor(false);
				break;
				
			case NP_SCORE_FOUR:
				currentGame = new ScoreFourCube ();
				window_showCursor(true);
				window_captureCursor(false);
				break;
				
			case NP_KEPLER_BUBBLE:
				currentGame = new KeplerBubble ();
				window_showCursor(false);
				window_captureCursor(true);
				break;				
		}
	
		currentNewtState = currentGame;
		stateName = NP_GAME;
	}
}


void NewtPuzzle::quitCurrentNewtState() {
	
	switch(stateName) {
		case NP_CHOOSER:
			window_quit();
		break;
			
		case NP_GAME:
			currentNewtState->quit();
			scene->clear();
			delete currentNewtState;
			stateName = NP_CHOOSER;
			chooser = new GameChooser();
			currentNewtState = chooser;	
			currentGame = NULL;
			window_showCursor(false);
			window_captureCursor(true);
			break;
			
		case NP_MENU:		
			
			break;
	}
}


bool NewtPuzzle::keyboardEvent(SDLKey key, double timeInterval) {
	
	switch(stateName) {
		case NP_CHOOSER:
			if(key == currentNewtState->getKeyboardOptions()->getKey("launch_key")) {
				launch (chooser->getSelectedLauncher());
				return false;
				
			} else {
				return currentNewtState-> keyboardEvent(key, timeInterval);
			}
			break;
			
		case NP_GAME:
			return currentNewtState-> keyboardEvent(key, timeInterval);
			break;
			
		case NP_MENU:

			break;		
	
		
	}
	
	return false;
}


